#!/bin/bash
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.

PKG_NAME="GitLab"
BACKUP_CONFIG="/usr/syno/etc/packages/Docker-GitLab/config"
POSTGRESQL_VERSION="0051"

need_restore()
{
	if [ -f "$BACKUP_CONFIG" ]; then
		return 0
	else
		return 1
	fi
}

memory_lt()
{
	mem_total=$(/bin/free -m | grep Mem |awk '{print $2}')
	if [ "$mem_total" -lt "$1" ]; then
		return 0
	else
		return 1
	fi
}

need_convert_db()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -lt "$POSTGRESQL_VERSION" ]; then
		return 0
	fi
	return 1
}

page_append()
{
	if [ -z "$1" ]; then
		echo "$2"
	elif [ -z "$2" ]; then
		echo "$1"
	else
		echo "$1,$2"
	fi
}

get_share()
{
	get_key_value "$BACKUP_CONFIG" SHARE
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}
wizard_found_backup="Velg en metode for å importere data."
wizard_decide_restore="GitLab-databasen eksisterer allerede.<br>Velg en av følgende handlinger:"
wizard_restore="Bruk eksisterende data"
wizard_create_new="Ren installasjon (alle eksisterende data, inkludert konfigurasjonsfiler og databasen, blir fjernet.)"
install_title="Installer GitLab"
install_data_root_desc="Opprett en delt mappe for å lagre dataene for GitLab."
install_data_root_label="Navn på delte mapper"
http_port_desc="Angi HTTP-portnummeret for GitLab."
http_port_label="HTTP-portnummer"
ssh_port_desc="Angi SSH-portnummeret for GitLab."
ssh_port_label="SSH-portnummer"
hostname_label="Domenenavn"
admin_email_desc="Åpne e-postkontoen til GitLab-administrator."
admin_email_label="e-post"
smtp_enable_desc="Aktiver GitLab for å sende e-poster via en SMTP-server."
smtp_enable_label="Aktiver SMTP"
smtp_address_desc="Angi SMTP-serveradressen som verter GitLab."
smtp_address_label="SMTP-adresse"
smtp_port_desc="Angi SMTP-serverporten."
smtp_port_label="SMTP-port"
smtp_user_desc="Angi SMTP-serverbrukernavnet."
smtp_user_label="SMTP-brukernavn"
smtp_pass_desc="Angi SMTP-serverpassordet."
smtp_pass_label="SMTP-passord"
smtp_verify_desc="Når GitLab er tilkoblet, verifiserer du SMTP-serverens SSL-sertifikat."
smtp_verify_label="Verifiser SSL-sertifikatet og avslå usikre tilkoblinger."
default_gitlab_account_desc="Standard innloggingsinformasjon er root/5iveL!fe"
db_connect_error="Kunne ikke koble til GitLab. Kontroller at databasen kjører som normalt."
mem_total_check_fail="Minimum {1} RAM kreves for å installere eller oppdatere <a href=\"{2}\" target=\"_blank\">den nyeste versjonen av GitLab</a>. Utvid minnet på Synology NAS-enheten din og prøv på nytt."
mem_total_check_warning="Minimum {1} RAM kreves for å installere eller oppdatere GitLab. Utilstrekkelig minne kan føre til uventede feil når du kjører GitLab, så vi anbefaler at du utvider minnet til Synology NAS-enheten din. <a href=\"{2}\" target=\"_blank\">Finn ut mer</a>."
upgrade_warning_page_title="Oppgrader Docker-GitLab"
postgresql_warning_page_desc="Den nye versjonen av Docker-GitLab erstatter MariaDB med <a href=\"https://docs.gitlab.com/ce/install/requirements.html#database\" target=\"_blank\">PostgreSQL</a> som bakdatabase."
upgrade_agree="Jeg godtar å oppgradere og konvertere databasen til PostgreSQL."
PageRestore()
{
	[ ! -z "$1" ] && local warning_page_desc="$1"
cat << EOF
{
	"step_title": "$wizard_found_backup",
	"items": [{
		"type": "singleselect",
		"desc": "$wizard_decide_restore",
		"subitems": [{
			"key": "restore_backup",
			"desc": "${wizard_restore}${warning_page_desc}",
			"defaultValue": true
		}, {
			"desc": "$wizard_create_new",
			"defaultValue": false
		}]
	}, {
		"type": "textfield",
		"subitems": [{
			"key": "worker_mode",
			"desc": "drop or skip",
			"defaultValue": "skip",
			"hidden": true
		}]
	}]
}
EOF
}

PageRestorePostgresql()
{
	PageRestore "<br>$(echo "$postgresql_warning_page_desc" | quote_json)"
}
//in DSM 6.x
CheckRestoreV1()
{
cat << EOF
// find constructor contains restore page
for (i = arguments[0].ownerCt.items.length-1; i >= 1; i--){
	page = arguments[0].ownerCt.items.items[i];
	if (page.headline === \"${wizard_found_backup}\"){
		// check whether user wants to restore or not
		restore = page.items.items[1].checked;
		break;
	}
}
EOF
}

//Wizard add step container in DSM 7.0.
CheckRestoreV2()
{
cat << EOF
// find constructor contains restore page
for (i = arguments[0].ownerCt.ownerCt.items.length-1; i >= 1; i--){
	page = arguments[0].ownerCt.ownerCt.items.items[i].getStep();
	if (page.headline === \"${wizard_found_backup}\"){
		// check whether user wants to restore or not
		restore = page.items.items[1].checked;
		break;
	}
}
EOF
}

CheckRestore()
{
	dsm_build_num=$(/bin/uname -v | cut -d' ' -f1 | sed -e "s|#||g")
	if [ "$dsm_build_num" -ge 40000 ]; then
		CheckRestoreV2
	else
		CheckRestoreV1
	fi
}

FindObj()
{
cat << EOF
for (i = 0; i < arguments[0].items.length; i++) {
	item = arguments[0].items.items[i]
	if (\"${1}\" === item.itemId){
		$2 = arguments[0].items.items[i];
		break;
	}
}
EOF
}

Activate()
{
cat << EOF
	"activeate": "{
		restore = $DEFAULT_RESTORE; // user restore or not, set by is_restore
		// will set restore
		$(CheckRestore)

		if (arguments[0].items.items[0].value === \"${install_data_root_desc}\") {
			dataroot_textfield = null;
			$(FindObj "pkgwizard_dataroot" "dataroot_textfield")
			if (restore) {
				dataroot_textfield.setValue(\"@OLD_DATAROOT@\");
			}
			dataroot_textfield.setDisabled(restore);
		}
	}",
EOF
}

ApplyDataRootInfo()
{
	local page="$1"
	sed "s/@OLD_DATAROOT@/$OLD_DATAROOT/g" <<< "$page"
}

PageInstallSetting()
{
	local page=$(cat << EOF
{
	"step_title": "$install_title",
	"items": [{
		"type": "textfield",
		"desc": "$install_data_root_desc",
		"subitems": [{
			"key": "pkgwizard_dataroot",
			"desc": "$install_data_root_label",
			"defaultValue": "gitlab",
			"validator": {
				"allowBlank": false
			}
		}]
	},{
		"type": "textfield",
		"desc": "$http_port_desc",
		"subitems": [{
			"key": "pkgwizard_http_port",
			"desc": "$http_port_label",
			"defaultValue": "30000",
			"validator": {
				"allowBlank": false,
				"regex": {
					"expr": "/^[1-9]\\\\d{0,4}$/"
				}
			}
		}]
	},{
		"type": "textfield",
		"desc": "$ssh_port_desc",
		"subitems": [{
			"key": "pkgwizard_ssh_port",
			"desc": "$ssh_port_label",
			"defaultValue": "30001",
			"validator": {
				"allowBlank": false,
				"regex": {
					"expr": "/^[1-9]\\\\d{0,4}$/"
				}
			}
		}]
	}]
},{
	"step_title": "$install_title",
	"items": [{
		"type": "textfield",
		"desc": "$hostname_desc",
		"subitems": [{
			"key": "pkgwizard_hostname",
			"desc": "$hostname_label",
			"defaultValue": "localhost",
			"validator": {
				"allowBlank": false
			}
		}]
	},{
		"type": "textfield",
		"desc": "$admin_email_desc",
		"subitems": [{
			"key": "pkgwizard_admin_email",
			"desc": "$admin_email_label",
			"defaultValue": "example@example.com",
			"validator": {
				"allowBlank": false,
				"vtype": "email"
			}
		}]
	}]
},{
	"step_title": "$install_title",
	$(Activate)
	"items": [{
		"type": "multiselect",
		"desc": "$smtp_enable_desc",
		"subitems": [{
			"key": "pkgwizard_smtp_enable",
			"desc": "$smtp_enable_label",
			"defaultValue": false
		}]
	},{
		"type": "textfield",
		"desc": "$smtp_address_desc",
		"subitems": [{
			"key": "pkgwizard_smtp_address",
			"desc": "$smtp_address_label"
		}]
	},{
		"type": "textfield",
		"desc": "$smtp_port_desc",
		"subitems": [{
			"key": "pkgwizard_smtp_port",
			"desc": "$smtp_port_label",
			"defaultValue": "587",
			"validator": {
				"allowBlank": false,
				"regex": {
					"expr": "/^[1-9]\\\\d{0,4}$/"
				}
			}
		}]
	},{
		"type": "textfield",
		"desc": "$smtp_user_desc",
		"subitems": [{
			"key": "pkgwizard_smtp_user",
			"desc": "$smtp_user_label"
		}]
	},{
		"type": "password",
		"desc": "$smtp_pass_desc",
		"subitems": [{
			"key": "pkgwizard_smtp_pass",
			"desc": "$smtp_pass_label"
		}]
	},{
		"type": "multiselect",
		"desc": "$smtp_verify_desc",
		"subitems": [{
			"key": "pkgwizard_smtp_verify",
			"desc": "$smtp_verify_label",
			"defaultValue": true
		}]
	}]
}
EOF
)
	ApplyDataRootInfo "$page"
}
PageMemoryCheck()
{
	min_mem="4GB"
	gitlab_mem_requirement_link=https://gitlab.com/gitlab-org/gitlab-ce/blob/v11.11.8/doc/install/requirements.md#memory
	mem_total_check_fail=${mem_total_check_fail//\{1\}/"$min_mem"}
	mem_total_check_fail=${mem_total_check_fail//\{2\}/"$gitlab_mem_requirement_link"}
	mem_total_check_warning=${mem_total_check_warning//\{1\}/$min_mem}
	mem_total_check_warning=${mem_total_check_warning//\{2\}/"$gitlab_mem_requirement_link"}
	is_soft_check=$1
	if $is_soft_check; then
		desc="$(echo $mem_total_check_warning | quote_json)"
	else
		desc="$(echo $mem_total_check_fail | quote_json)"
	fi
cat << EOF
{
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "textfield",
		"desc": "$desc",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return $is_soft_check;}"
			} 
		}]
	}]
}
EOF
}
main()
{
	local install_page=""
	local memory_check_page=""
	local restore_page=""
	local install_setting_page=""

	DEFAULT_RESTORE=false
	OLD_DATAROOT="$(get_share)"

	# 2GB and 4GB check
	if memory_lt 1800; then
		memory_check_page="$(PageMemoryCheck false)"
	elif memory_lt 3800; then
		memory_check_page="$(PageMemoryCheck true)"
	fi

	if need_restore; then
		. "$BACKUP_CONFIG"
		if [ ! -z $VERSION ] && need_convert_db "$VERSION"; then
			restore_page="$(PageRestorePostgresql)"
		else
			restore_page="$(PageRestore)"
		fi
	fi
	install_setting_page="$(PageInstallSetting)"

	install_page=$(page_append "$install_page" "$memory_check_page")
	install_page=$(page_append "$install_page" "$restore_page")
	install_page=$(page_append "$install_page" "$install_setting_page")
	echo "[$install_page]" > "${SYNOPKG_TEMP_LOGFILE}"
	return 0
}

main "$@"
