#!/bin/bash
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.

PKG_NAME="GitLab"
BACKUP_CONFIG="/usr/syno/etc/packages/Docker-GitLab/config"
POSTGRESQL_VERSION="0051"

need_restore()
{
	if [ -f "$BACKUP_CONFIG" ]; then
		return 0
	else
		return 1
	fi
}

memory_lt()
{
	mem_total=$(/bin/free -m | grep Mem |awk '{print $2}')
	if [ "$mem_total" -lt "$1" ]; then
		return 0
	else
		return 1
	fi
}

need_convert_db()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -lt "$POSTGRESQL_VERSION" ]; then
		return 0
	fi
	return 1
}

page_append()
{
	if [ -z "$1" ]; then
		echo "$2"
	elif [ -z "$2" ]; then
		echo "$1"
	else
		echo "$1,$2"
	fi
}

get_share()
{
	get_key_value "$BACKUP_CONFIG" SHARE
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}
wizard_found_backup="請選擇資料匯入方式。"
wizard_decide_restore="GitLab 資料庫已存在。<br>請選擇下方任一動作："
wizard_restore="使用原有資料"
wizard_create_new="全新安裝 (包含設定檔及資料庫的原有資料將全部被移除。)"
install_title="安裝 GitLab"
install_data_root_desc="建立共用資料夾來儲存 GitLab 的資料。"
install_data_root_label="共用資料夾名稱"
http_port_desc="請輸入 GitLab 使用的 HTTP 連接埠編號。 "
http_port_label="HTTP 連接埠號"
ssh_port_desc="請輸入 GitLab 使用的 SSH 連接埠號。"
ssh_port_label="SSH 連接埠號"
hostname_label="網域名稱"
admin_email_desc="請輸入 GitLab 的管理員的 email 帳號。"
admin_email_label="email"
smtp_enable_desc="讓 GitLab 能透過 SMTP 伺服機寄送 email。"
smtp_enable_label="啟用 SMTP"
smtp_address_desc="請輸入 GitLab 所使用的 SMTP 伺服機位址。"
smtp_address_label="SMTP 位址"
smtp_port_desc="請輸入 SMTP 伺服機的連接埠。"
smtp_port_label="SMTP 連接埠"
smtp_user_desc="請輸入 SMTP 伺服機的使用者帳號。"
smtp_user_label="SMTP 使用者帳號"
smtp_pass_desc="請輸入 SMTP 伺服機的密碼。"
smtp_pass_label="SMTP 密碼"
smtp_verify_desc="當 GitLab 與 SMTP 伺服機連線時，驗證其 SSL 憑證。"
smtp_verify_label="驗證 SSL 憑證並拒絕不安全的連線。"
default_gitlab_account_desc="預設登入帳號密碼是「root/5iveL!fe」。"
db_connect_error="GitLab 連線失敗，請確認資料庫正常運行。"
mem_total_check_fail="安裝或更新至<a href=\"{2}\" target=\"_blank\">最新版本 GitLab </a>至少需要 {1} 的記憶體。請擴充你的 Synology NAS 記憶體容量後再試一次。"
mem_total_check_warning="安裝或更新至最新版 GitLab 至少需要 {1} 的記憶體。記憶體不足將可能造成執行 GitLab 時發生不預期錯誤，因此建議你擴充你的 Synology NAS 記憶體容量。<a href=\"{2}\" target=\"_blank\">了解更多</a>。"
upgrade_warning_page_title="升級 Docker-GitLab"
postgresql_warning_page_desc="新版本 Docker-GitLab 將使用 <a href=\"https://docs.gitlab.com/ce/install/requirements.html#database\" target=\"_blank\">PostgreSQL</a> 取代 MariaDB 作為後端資料庫。"
upgrade_agree="我同意升級並將資料庫轉換至 PostgreSQL。"
PageMemoryCheck()
{
	min_mem="4GB"
	gitlab_mem_requirement_link=https://gitlab.com/gitlab-org/gitlab-ce/blob/v11.11.8/doc/install/requirements.md#memory
	mem_total_check_fail=${mem_total_check_fail//\{1\}/"$min_mem"}
	mem_total_check_fail=${mem_total_check_fail//\{2\}/"$gitlab_mem_requirement_link"}
	mem_total_check_warning=${mem_total_check_warning//\{1\}/$min_mem}
	mem_total_check_warning=${mem_total_check_warning//\{2\}/"$gitlab_mem_requirement_link"}
	is_soft_check=$1
	if $is_soft_check; then
		desc="$(echo $mem_total_check_warning | quote_json)"
	else
		desc="$(echo $mem_total_check_fail | quote_json)"
	fi
cat << EOF
{
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "textfield",
		"desc": "$desc",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return $is_soft_check;}"
			} 
		}]
	}]
}
EOF
}
PagePostegresqlWarning()
{
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "multiselect",
		"desc": "$(echo "$postgresql_warning_page_desc" | quote_json)",
		"subitems": [{
			"key": "pkgwizard_agree",
			"desc": "$upgrade_agree",
			"defaultValue": false,
			"validator": {
				"fn": "{return arguments[0];}"
			}
		}]
	}]
}
EOF
}
main()
{
	local upgrade_page=""
	local memory_check_page=""
	local postgresql_warning_page=""

	DEFAULT_RESTORE=true
	local version="$(echo "$SYNOPKG_OLD_PKGVER" | cut -d- -f2)"

	# 2GB and 4GB check
	if memory_lt 1800; then
		memory_check_page="$(PageMemoryCheck false)"
	elif memory_lt 3800; then
		memory_check_page="$(PageMemoryCheck true)"
	fi

	if need_convert_db "$version"; then
		postgresql_warning_page="$(PagePostegresqlWarning)"
	fi

	upgrade_page=$(page_append "$upgrade_page" "$memory_check_page")
	upgrade_page=$(page_append "$upgrade_page" "$postgresql_warning_page")
	echo "[$upgrade_page]" > "${SYNOPKG_TEMP_LOGFILE}"
	return 0
}

main "$@"
