#!/bin/bash
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.

PKG_NAME="GitLab"
BACKUP_CONFIG="/usr/syno/etc/packages/Docker-GitLab/config"
POSTGRESQL_VERSION="0051"

need_restore()
{
	if [ -f "$BACKUP_CONFIG" ]; then
		return 0
	else
		return 1
	fi
}

memory_lt()
{
	mem_total=$(/bin/free -m | grep Mem |awk '{print $2}')
	if [ "$mem_total" -lt "$1" ]; then
		return 0
	else
		return 1
	fi
}

need_convert_db()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -lt "$POSTGRESQL_VERSION" ]; then
		return 0
	fi
	return 1
}

page_append()
{
	if [ -z "$1" ]; then
		echo "$2"
	elif [ -z "$2" ]; then
		echo "$1"
	else
		echo "$1,$2"
	fi
}

get_share()
{
	get_key_value "$BACKUP_CONFIG" SHARE
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}
wizard_found_backup="Sélectionnez une méthode d'importation des données."
wizard_decide_restore="La base de données GitLab existe déjà.<br>Sélectionnez l'une des actions suivantes :"
wizard_restore="Utiliser les données existantes"
wizard_create_new="Nettoyer l'installation (Toutes les données existantes, notamment les fichiers de configuration et la base de données, seront supprimés.)"
install_title="Installer GitLab"
install_data_root_desc="Créez un dossier partagé pour stocker les données de GitLab."
install_data_root_label="Nom de dossier partagé"
http_port_desc="Veuillez saisir le numéro du port HTTP pour GitLab."
http_port_label="Numéro du port HTTP"
ssh_port_desc="Veuillez saisir le numéro du port SSH pour GitLab."
ssh_port_label="Numéro du port SSH"
hostname_label="Nom de domaine"
admin_email_desc="Saisissez le compte de messagerie administrateur GitLab."
admin_email_label="e-mail"
smtp_enable_desc="Activez GitLab pour envoyer des e-mails via un serveur SMTP."
smtp_enable_label="Activer SMTP"
smtp_address_desc="Saisissez l'adresse du serveur SMTP qui héberge GitLab."
smtp_address_label="Adresse SMTP"
smtp_port_desc="Saisissez le numéro de port du serveur SMTP."
smtp_port_label="Port SMTP"
smtp_user_desc="Saisissez le nom d'utilisateur du serveur SMTP."
smtp_user_label="Nom d'utilisateur SMTP"
smtp_pass_desc="Saisissez le mot de passe du serveur SMTP."
smtp_pass_label="Mot de passe SMTP"
smtp_verify_desc="Lorsque GitLab est connecté, vérifiez le certificat SSL du serveur SMTP."
smtp_verify_label="Vérifiez le certificat SSL et refuser les connexions non fiables"
default_gitlab_account_desc="L'identifiant de connexion par défaut est root/5iveL!fe"
db_connect_error="Échec de la connexion à GitLab. Assurez-vous que la base de données fonctionne correctement."
mem_total_check_fail="Un minimum de {1} de RAM est nécessaire pour installer ou mettre à jour <a href=\"{2}\" target=\"_blank\">la dernière version de GitLab</a>. Augmentez la mémoire de votre Synology NAS et réessayez."
mem_total_check_warning="Un minimum de {1} de RAM est nécessaire pour installer ou mettre à jour GitLab. Une mémoire insuffisante peut provoquer des erreurs inattendues lorsque vous lancez GitLab ; c'est pourquoi nous vous recommandons d'augmenter la mémoire de votre Synology NAS. <a href=\"{2}\" target=\"_blank\">En savoir plus</a>."
upgrade_warning_page_title="Mettre à niveau Docker-GitLab"
postgresql_warning_page_desc="La nouvelle version de Docker-GitLab va remplacer MariaDB par <a href=\"https://docs.gitlab.com/ce/install/requirements.html#database\" target=\"_blank\">PostgreSQL</a> comme base de données principale."
upgrade_agree="J'accepte de procéder à la mise à niveau et de convertir la base de données en PostgreSQL."
PageMemoryCheck()
{
	min_mem="4GB"
	gitlab_mem_requirement_link=https://gitlab.com/gitlab-org/gitlab-ce/blob/v11.11.8/doc/install/requirements.md#memory
	mem_total_check_fail=${mem_total_check_fail//\{1\}/"$min_mem"}
	mem_total_check_fail=${mem_total_check_fail//\{2\}/"$gitlab_mem_requirement_link"}
	mem_total_check_warning=${mem_total_check_warning//\{1\}/$min_mem}
	mem_total_check_warning=${mem_total_check_warning//\{2\}/"$gitlab_mem_requirement_link"}
	is_soft_check=$1
	if $is_soft_check; then
		desc="$(echo $mem_total_check_warning | quote_json)"
	else
		desc="$(echo $mem_total_check_fail | quote_json)"
	fi
cat << EOF
{
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "textfield",
		"desc": "$desc",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return $is_soft_check;}"
			} 
		}]
	}]
}
EOF
}
PagePostegresqlWarning()
{
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "multiselect",
		"desc": "$(echo "$postgresql_warning_page_desc" | quote_json)",
		"subitems": [{
			"key": "pkgwizard_agree",
			"desc": "$upgrade_agree",
			"defaultValue": false,
			"validator": {
				"fn": "{return arguments[0];}"
			}
		}]
	}]
}
EOF
}
main()
{
	local upgrade_page=""
	local memory_check_page=""
	local postgresql_warning_page=""

	DEFAULT_RESTORE=true
	local version="$(echo "$SYNOPKG_OLD_PKGVER" | cut -d- -f2)"

	# 2GB and 4GB check
	if memory_lt 1800; then
		memory_check_page="$(PageMemoryCheck false)"
	elif memory_lt 3800; then
		memory_check_page="$(PageMemoryCheck true)"
	fi

	if need_convert_db "$version"; then
		postgresql_warning_page="$(PagePostegresqlWarning)"
	fi

	upgrade_page=$(page_append "$upgrade_page" "$memory_check_page")
	upgrade_page=$(page_append "$upgrade_page" "$postgresql_warning_page")
	echo "[$upgrade_page]" > "${SYNOPKG_TEMP_LOGFILE}"
	return 0
}

main "$@"
