#!/bin/bash
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.

PKG_NAME="GitLab"
BACKUP_CONFIG="/usr/syno/etc/packages/Docker-GitLab/config"
POSTGRESQL_VERSION="0051"

need_restore()
{
	if [ -f "$BACKUP_CONFIG" ]; then
		return 0
	else
		return 1
	fi
}

memory_lt()
{
	mem_total=$(/bin/free -m | grep Mem |awk '{print $2}')
	if [ "$mem_total" -lt "$1" ]; then
		return 0
	else
		return 1
	fi
}

need_convert_db()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -lt "$POSTGRESQL_VERSION" ]; then
		return 0
	fi
	return 1
}

page_append()
{
	if [ -z "$1" ]; then
		echo "$2"
	elif [ -z "$2" ]; then
		echo "$1"
	else
		echo "$1,$2"
	fi
}

get_share()
{
	get_key_value "$BACKUP_CONFIG" SHARE
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}
wizard_found_backup="Bitte wählen Sie eine Methode, um Daten zu importieren."
wizard_decide_restore="GitLab-Datenbank ist bereits vorhanden.<br>Bitte wählen Sie eine der folgenden Aktionen aus:"
wizard_restore="Vorhandene Daten verwenden"
wizard_create_new="Installation reinigen (sämtliche vorhandenen Daten, einschließlich Konfigurationsdateien und die Datenbank, werden entfernt.)"
install_title="GitLab installieren"
install_data_root_desc="Erstellen Sie einen gemeinsamen Ordner zum Speichern der Daten von GitLab."
install_data_root_label="Name des gemeinsamen Ordners"
http_port_desc="Geben Sie die HTTP-Portnummer für GitLab ein."
http_port_label="HTTP-Portnummer"
ssh_port_desc="Geben Sie die SSH-Portnummer für GitLab ein."
ssh_port_label="SSH-Portnummer"
hostname_label="Domainname"
admin_email_desc="Geben Sie das E-Mail-Konto des GitLab-Administrators ein."
admin_email_label="E-Mail"
smtp_enable_desc="Aktivieren Sie GitLab, um E-Mails über einen SMTP-Server zu senden."
smtp_enable_label="SMTP aktivieren"
smtp_address_desc="Geben Sie die SMTP-Serveradresse von GitLab ein."
smtp_address_label="SMTP-Adresse"
smtp_port_desc="Geben Sie den SMTP-Serverport ein."
smtp_port_label="SMTP-Port"
smtp_user_desc="Geben Sie den Benutzernamen des SMTP-Servers ein."
smtp_user_label="SMTP-Benutzername"
smtp_pass_desc="Geben Sie das Kennwort für den SMTP-Server ein."
smtp_pass_label="SMTP-Kennwort"
smtp_verify_desc="Überprüfen Sie, wenn GitLab verbunden ist, das SSL-Zertifikat des SMTP-Servers."
smtp_verify_label="Überprüfen Sie das SSL-Zertifikat und weisen Sie unsichere Verbindungen zurück."
default_gitlab_account_desc="Der Standard-Anmeldename ist root/5iveL!fe"
db_connect_error="Verbindung zu GitLab fehlgeschlagen. Stellen Sie sicher, dass die Datenbank ordnungsgemäß läuft."
mem_total_check_fail="Mindestens {1} RAM ist erforderlich, um <a href=\"{2}\" target=\"_blank\">die neueste Version von GitLab</a> zu installieren oder zu aktualisieren. Erweitern Sie den Speicher Ihres Synology NAS und versuchen Sie es erneut."
mem_total_check_warning="Mindestens {1} RAM ist erforderlich, um GitLab zu installieren oder zu aktualisieren. Nicht ausreichender Speicher kann zu unerwarteten Fehlern bei der Ausführung von GitLab führen, daher sollten Sie den Speicher Ihres Synology NAS erweitern. <a href=\"{2}\" target=\"_blank\">Weitere Informationen</a>."
upgrade_warning_page_title="Docker-GitLab aktualisieren"
postgresql_warning_page_desc="Die neue Version von Docker-GitLab ersetzt MariaDB durch <a href=\"https://docs.gitlab.com/ce/install/requirements.html#database\" target=\"_blank\">PostgreSQL</a> als Back-End-Datenbank."
upgrade_agree="Ich stimme dem Upgrade und der Konvertierung der Datenbank auf PostgreSQL zu."
PageMemoryCheck()
{
	min_mem="4GB"
	gitlab_mem_requirement_link=https://gitlab.com/gitlab-org/gitlab-ce/blob/v11.11.8/doc/install/requirements.md#memory
	mem_total_check_fail=${mem_total_check_fail//\{1\}/"$min_mem"}
	mem_total_check_fail=${mem_total_check_fail//\{2\}/"$gitlab_mem_requirement_link"}
	mem_total_check_warning=${mem_total_check_warning//\{1\}/$min_mem}
	mem_total_check_warning=${mem_total_check_warning//\{2\}/"$gitlab_mem_requirement_link"}
	is_soft_check=$1
	if $is_soft_check; then
		desc="$(echo $mem_total_check_warning | quote_json)"
	else
		desc="$(echo $mem_total_check_fail | quote_json)"
	fi
cat << EOF
{
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "textfield",
		"desc": "$desc",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return $is_soft_check;}"
			} 
		}]
	}]
}
EOF
}
PagePostegresqlWarning()
{
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "multiselect",
		"desc": "$(echo "$postgresql_warning_page_desc" | quote_json)",
		"subitems": [{
			"key": "pkgwizard_agree",
			"desc": "$upgrade_agree",
			"defaultValue": false,
			"validator": {
				"fn": "{return arguments[0];}"
			}
		}]
	}]
}
EOF
}
main()
{
	local upgrade_page=""
	local memory_check_page=""
	local postgresql_warning_page=""

	DEFAULT_RESTORE=true
	local version="$(echo "$SYNOPKG_OLD_PKGVER" | cut -d- -f2)"

	# 2GB and 4GB check
	if memory_lt 1800; then
		memory_check_page="$(PageMemoryCheck false)"
	elif memory_lt 3800; then
		memory_check_page="$(PageMemoryCheck true)"
	fi

	if need_convert_db "$version"; then
		postgresql_warning_page="$(PagePostegresqlWarning)"
	fi

	upgrade_page=$(page_append "$upgrade_page" "$memory_check_page")
	upgrade_page=$(page_append "$upgrade_page" "$postgresql_warning_page")
	echo "[$upgrade_page]" > "${SYNOPKG_TEMP_LOGFILE}"
	return 0
}

main "$@"
