#!/bin/bash
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.

PKG_NAME="GitLab"
BACKUP_CONFIG="/usr/syno/etc/packages/Docker-GitLab/config"
POSTGRESQL_VERSION="0051"

need_restore()
{
	if [ -f "$BACKUP_CONFIG" ]; then
		return 0
	else
		return 1
	fi
}

memory_lt()
{
	mem_total=$(/bin/free -m | grep Mem |awk '{print $2}')
	if [ "$mem_total" -lt "$1" ]; then
		return 0
	else
		return 1
	fi
}

need_convert_db()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -lt "$POSTGRESQL_VERSION" ]; then
		return 0
	fi
	return 1
}

page_append()
{
	if [ -z "$1" ]; then
		echo "$2"
	elif [ -z "$2" ]; then
		echo "$1"
	else
		echo "$1,$2"
	fi
}

get_share()
{
	get_key_value "$BACKUP_CONFIG" SHARE
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}
wizard_found_backup="データをインポートする方法を選択してください。"
wizard_decide_restore="GitLab データベースはすでに存在します。<br>以下のいずれかの操作を選択してください。"
wizard_restore="既存のデータを使用"
wizard_create_new="クリーン インストール (構成ファイルやデータベースなどすべての既存のデータが削除されます。)"
install_title="GitLab をインストール"
install_data_root_desc="GitLab のデータを保管しておくための共有フォルダを作成します。"
install_data_root_label="共有フォルダ名"
http_port_desc="GitLab の HTTP ポート番号を入力してください。"
http_port_label="HTTP ポート番号"
ssh_port_desc="GitLab のSSH ポート番号を入力してください。"
ssh_port_label="SSH ポート番号"
hostname_label="ドメイン名"
admin_email_desc="GitLab 管理者の E メール アカウントを入力してください。"
admin_email_label="Eメール"
smtp_enable_desc="SMTP サーバーから E メールを送信するには、GitLab を有効にしてください。"
smtp_enable_label="SMTP を有効にする"
smtp_address_desc="GitLab をホストする SMTP サーバー アドレスを入力します。"
smtp_address_label="SMTP アドレス"
smtp_port_desc="SMTP サーバーポートを入力してください。"
smtp_port_label="SMTP ポート"
smtp_user_desc="SMTP サーバー ユーザー名を入力してください。"
smtp_user_label="SMTP ユーザー名"
smtp_pass_desc="SMTP サーバー パスワードを入力してください。"
smtp_pass_label="SMTP パスワード"
smtp_verify_desc="GitLab が接続している場合は、確実に SMTP サーバーの SSL 証明書を確認してください。"
smtp_verify_label="SSL 証明書を確認し、安全ではない接続は拒否してください。"
default_gitlab_account_desc="デフォルトのログイン情報は root/5iveL!fe です。"
db_connect_error="GitLab への接続に失敗しました。 データベースが適切に動作しているか確認してください。"
mem_total_check_fail="最小{1} RAM が<a href=\"{2}\" target=\"_blank\">GitLab の最新バージョン</a>のインストールまたはアップデートに必要です。 Synology NASのメモリを拡張して再試行してください。"
mem_total_check_warning="最小{1} RAM が GitLab のインストールまたはアップデートに必要です。 メモリが不足していると、GitLab を実行中に予期せぬエラーが起きる原因になることがあります。ですので、Synology NASのメモリを拡張することをお勧めします。 <a href=\"{2}\" target=\"_blank\">詳細</a>。"
upgrade_warning_page_title="Docker-GitLabをアップグレード"
postgresql_warning_page_desc="Docker-GitLabの新しいバージョンがバックエンド データベースとしてMariaDBを<a href=\"https://docs.gitlab.com/ce/install/requirements.html#database\" target=\"_blank\">PostgreSQL</a>と交換します。"
upgrade_agree="私は、アップグレードに同意し、データベースをPostgreSQLに変換します。"
PageMemoryCheck()
{
	min_mem="4GB"
	gitlab_mem_requirement_link=https://gitlab.com/gitlab-org/gitlab-ce/blob/v11.11.8/doc/install/requirements.md#memory
	mem_total_check_fail=${mem_total_check_fail//\{1\}/"$min_mem"}
	mem_total_check_fail=${mem_total_check_fail//\{2\}/"$gitlab_mem_requirement_link"}
	mem_total_check_warning=${mem_total_check_warning//\{1\}/$min_mem}
	mem_total_check_warning=${mem_total_check_warning//\{2\}/"$gitlab_mem_requirement_link"}
	is_soft_check=$1
	if $is_soft_check; then
		desc="$(echo $mem_total_check_warning | quote_json)"
	else
		desc="$(echo $mem_total_check_fail | quote_json)"
	fi
cat << EOF
{
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "textfield",
		"desc": "$desc",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return $is_soft_check;}"
			} 
		}]
	}]
}
EOF
}
PagePostegresqlWarning()
{
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "multiselect",
		"desc": "$(echo "$postgresql_warning_page_desc" | quote_json)",
		"subitems": [{
			"key": "pkgwizard_agree",
			"desc": "$upgrade_agree",
			"defaultValue": false,
			"validator": {
				"fn": "{return arguments[0];}"
			}
		}]
	}]
}
EOF
}
main()
{
	local upgrade_page=""
	local memory_check_page=""
	local postgresql_warning_page=""

	DEFAULT_RESTORE=true
	local version="$(echo "$SYNOPKG_OLD_PKGVER" | cut -d- -f2)"

	# 2GB and 4GB check
	if memory_lt 1800; then
		memory_check_page="$(PageMemoryCheck false)"
	elif memory_lt 3800; then
		memory_check_page="$(PageMemoryCheck true)"
	fi

	if need_convert_db "$version"; then
		postgresql_warning_page="$(PagePostegresqlWarning)"
	fi

	upgrade_page=$(page_append "$upgrade_page" "$memory_check_page")
	upgrade_page=$(page_append "$upgrade_page" "$postgresql_warning_page")
	echo "[$upgrade_page]" > "${SYNOPKG_TEMP_LOGFILE}"
	return 0
}

main "$@"
