#!/bin/bash
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.

PKG_NAME="GitLab"
BACKUP_CONFIG="/usr/syno/etc/packages/Docker-GitLab/config"
POSTGRESQL_VERSION="0051"

need_restore()
{
	if [ -f "$BACKUP_CONFIG" ]; then
		return 0
	else
		return 1
	fi
}

memory_lt()
{
	mem_total=$(/bin/free -m | grep Mem |awk '{print $2}')
	if [ "$mem_total" -lt "$1" ]; then
		return 0
	else
		return 1
	fi
}

need_convert_db()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -lt "$POSTGRESQL_VERSION" ]; then
		return 0
	fi
	return 1
}

page_append()
{
	if [ -z "$1" ]; then
		echo "$2"
	elif [ -z "$2" ]; then
		echo "$1"
	else
		echo "$1,$2"
	fi
}

get_share()
{
	get_key_value "$BACKUP_CONFIG" SHARE
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}
wizard_found_backup="Выберите способ импорта данных."
wizard_decide_restore="GitLab База данных уже существует.<br>Выберите одно из следующих действий."
wizard_restore="Использовать существующие данные"
wizard_create_new="Чистая установка (все имеющиеся данные, включая файлы конфигурации и базу данных, будут удалены)."
install_title="Установить GitLab"
install_data_root_desc="Создайте папку общего доступа для хранения данных GitLab."
install_data_root_label="Имя папки общего доступа"
http_port_desc="Введите номер порта HTTP для GitLab."
http_port_label="Номер порта HTTP"
ssh_port_desc="Введите номер порта SSH для GitLab."
ssh_port_label="Номер порта SSH"
hostname_label="Имя домена"
admin_email_desc="Введите учетную запись электронной почты администратора GitLab."
admin_email_label="электронная почта"
smtp_enable_desc="Включите GitLab для отправки сообщений эл. почты посредством сервера SMTP."
smtp_enable_label="Включить SMTP"
smtp_address_desc="Укажите адрес сервера SMTP, на котором размещен GitLab."
smtp_address_label="Адрес сервера SMTP"
smtp_port_desc="Введите порт сервера SMTP."
smtp_port_label="Порт сервера SMTP"
smtp_user_desc="Укажите имя пользователя сервера SMTP."
smtp_user_label="Имя пользователя сервера SMTP"
smtp_pass_desc="Введите пароль сервера SMTP."
smtp_pass_label="Пароль сервера SMTP"
smtp_verify_desc="При подключении к GitLab проверьте сертификат SSL сервера SMTP."
smtp_verify_label="Проверьте сертификат SSL и исключите небезопасные подключения."
default_gitlab_account_desc="Учетные данные по умолчанию — root/5iveL!fe"
db_connect_error="Не удалось подключиться к GitLab. Проверьте исправность базы данных."
mem_total_check_fail="Для установки или обновления <a href=\"{2}\" target=\"_blank\">последней версии GitLab</a> требуется как минимум {1} ОЗУ. Увеличьте объем памяти Synology NAS и повторите попытку."
mem_total_check_warning="Для установки или обновления GitLab требуется как минимум {1} ОЗУ. Недостаточный объем памяти может привести к непредвиденным ошибкам при запуске GitLab, поэтому рекомендуется увеличить объем памяти Synology NAS. <a href=\"{2}\" target=\"_blank\">Подробнее</a>."
upgrade_warning_page_title="Обновление Docker-GitLab"
postgresql_warning_page_desc="Новая версия Docker-GitLab заменит MariaDB. <a href=\"https://docs.gitlab.com/ce/install/requirements.html#database\" target=\"_blank\">PostgreSQL</a> будет выполнять роль внутренней базы данных."
upgrade_agree="Я согласен с обновлением и преобразованием базы данных в PostgreSQL."
PageMemoryCheck()
{
	min_mem="4GB"
	gitlab_mem_requirement_link=https://gitlab.com/gitlab-org/gitlab-ce/blob/v11.11.8/doc/install/requirements.md#memory
	mem_total_check_fail=${mem_total_check_fail//\{1\}/"$min_mem"}
	mem_total_check_fail=${mem_total_check_fail//\{2\}/"$gitlab_mem_requirement_link"}
	mem_total_check_warning=${mem_total_check_warning//\{1\}/$min_mem}
	mem_total_check_warning=${mem_total_check_warning//\{2\}/"$gitlab_mem_requirement_link"}
	is_soft_check=$1
	if $is_soft_check; then
		desc="$(echo $mem_total_check_warning | quote_json)"
	else
		desc="$(echo $mem_total_check_fail | quote_json)"
	fi
cat << EOF
{
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "textfield",
		"desc": "$desc",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return $is_soft_check;}"
			} 
		}]
	}]
}
EOF
}
PagePostegresqlWarning()
{
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "multiselect",
		"desc": "$(echo "$postgresql_warning_page_desc" | quote_json)",
		"subitems": [{
			"key": "pkgwizard_agree",
			"desc": "$upgrade_agree",
			"defaultValue": false,
			"validator": {
				"fn": "{return arguments[0];}"
			}
		}]
	}]
}
EOF
}
main()
{
	local upgrade_page=""
	local memory_check_page=""
	local postgresql_warning_page=""

	DEFAULT_RESTORE=true
	local version="$(echo "$SYNOPKG_OLD_PKGVER" | cut -d- -f2)"

	# 2GB and 4GB check
	if memory_lt 1800; then
		memory_check_page="$(PageMemoryCheck false)"
	elif memory_lt 3800; then
		memory_check_page="$(PageMemoryCheck true)"
	fi

	if need_convert_db "$version"; then
		postgresql_warning_page="$(PagePostegresqlWarning)"
	fi

	upgrade_page=$(page_append "$upgrade_page" "$memory_check_page")
	upgrade_page=$(page_append "$upgrade_page" "$postgresql_warning_page")
	echo "[$upgrade_page]" > "${SYNOPKG_TEMP_LOGFILE}"
	return 0
}

main "$@"
