#!/bin/bash
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.

PKG_NAME="GitLab"
BACKUP_CONFIG="/usr/syno/etc/packages/Docker-GitLab/config"
POSTGRESQL_VERSION="0051"

need_restore()
{
	if [ -f "$BACKUP_CONFIG" ]; then
		return 0
	else
		return 1
	fi
}

memory_lt()
{
	mem_total=$(/bin/free -m | grep Mem |awk '{print $2}')
	if [ "$mem_total" -lt "$1" ]; then
		return 0
	else
		return 1
	fi
}

need_convert_db()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -lt "$POSTGRESQL_VERSION" ]; then
		return 0
	fi
	return 1
}

page_append()
{
	if [ -z "$1" ]; then
		echo "$2"
	elif [ -z "$2" ]; then
		echo "$1"
	else
		echo "$1,$2"
	fi
}

get_share()
{
	get_key_value "$BACKUP_CONFIG" SHARE
}

quote_json() {
	sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}
wizard_found_backup="Seleccione un método para importar datos."
wizard_decide_restore="GitLab la base de datos ya existe.<br>Seleccione alguna de las siguientes acciones:"
wizard_restore="Utilizar los datos existentes"
wizard_create_new="Instalación limpia (se eliminarán todos los datos existentes, incluidos los archivos de configuración y la base de datos)."
install_title="Instalar GitLab"
install_data_root_desc="Cree una carpeta compartida para almacenar los datos de GitLab."
install_data_root_label="Nombre de carpeta compartida"
http_port_desc="Introduzca el número de puerto HTTP para GitLab."
http_port_label="Número de puerto HTTP"
ssh_port_desc="Introduzca el número de puerto SSH para GitLab."
ssh_port_label="Número de puerto SSH"
hostname_label="Nombre de dominio"
admin_email_desc="Introduzca la cuenta de correo electrónico del administrador GitLab."
admin_email_label="correo electrónico"
smtp_enable_desc="Habilite GitLab para enviar correos electrónicos a través de un servidor de SMTP."
smtp_enable_label="Habilitar SMTP"
smtp_address_desc="Introduzca el alojamiento de la dirección del servidor SMTP GitLab."
smtp_address_label="Dirección SMTP"
smtp_port_desc="Introduzca el puerto de servidor SMTP."
smtp_port_label="Puerto SMTP"
smtp_user_desc="Escriba el nombre de usuario del servidor SMTP."
smtp_user_label="Nombre de usuario SMTP"
smtp_pass_desc="Escriba la contraseña del servidor SMTP."
smtp_pass_label="Contraseña SMTP"
smtp_verify_desc="Cuando GitLab esté conectado, compruebe el certificado SSL del servidor SMTP."
smtp_verify_label="Compruebe el certificado SSL y rechace las conexiones que no sean seguras."
default_gitlab_account_desc="la credencial de inicio de sesión predeterminada es root/5iveL!fe"
db_connect_error="Error de conexión a GitLab. Asegúrese de que la base de datos se está ejecutando correctamente."
mem_total_check_fail="Se requiere un mínimo de {1} de RAM para instalar o actualizar <a href=\"{2}\" target=\"_blank\">la versión más reciente de GitLab</a>. Amplíe la memoria de su Synology NAS e inténtelo de nuevo."
mem_total_check_warning="Se requiere un mínimo de {1} de RAM para instalar o actualizar GitLab. Si la memoria es insuficiente, se pueden producir errores al ejecutar GitLab, por lo que es recomendable ampliar la memoria de su Synology NAS. <a href=\"{2}\" target=\"_blank\">Más información</a>."
upgrade_warning_page_title="Actualizar Docker-GitLab"
postgresql_warning_page_desc="La nueva versión de Docker-GitLab sustituirá a MariaDB con <a href=\"https://docs.gitlab.com/ce/install/requirements.html#database\" target=\"_blank\">PostgreSQL</a> como base de datos back-end."
upgrade_agree="Acepto actualizar y convertir la base de datos a PostgreSQL."
PageMemoryCheck()
{
	min_mem="4GB"
	gitlab_mem_requirement_link=https://gitlab.com/gitlab-org/gitlab-ce/blob/v11.11.8/doc/install/requirements.md#memory
	mem_total_check_fail=${mem_total_check_fail//\{1\}/"$min_mem"}
	mem_total_check_fail=${mem_total_check_fail//\{2\}/"$gitlab_mem_requirement_link"}
	mem_total_check_warning=${mem_total_check_warning//\{1\}/$min_mem}
	mem_total_check_warning=${mem_total_check_warning//\{2\}/"$gitlab_mem_requirement_link"}
	is_soft_check=$1
	if $is_soft_check; then
		desc="$(echo $mem_total_check_warning | quote_json)"
	else
		desc="$(echo $mem_total_check_fail | quote_json)"
	fi
cat << EOF
{
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "textfield",
		"desc": "$desc",
		"subitems": [{
			"hidden": true,
			"validator": {
				"fn": "{return $is_soft_check;}"
			} 
		}]
	}]
}
EOF
}
PagePostegresqlWarning()
{
cat << EOF
{
	"step_title": "$upgrade_warning_page_title",
	"invalid_next_disabled_v2": true,
	"items": [{
		"type": "multiselect",
		"desc": "$(echo "$postgresql_warning_page_desc" | quote_json)",
		"subitems": [{
			"key": "pkgwizard_agree",
			"desc": "$upgrade_agree",
			"defaultValue": false,
			"validator": {
				"fn": "{return arguments[0];}"
			}
		}]
	}]
}
EOF
}
main()
{
	local upgrade_page=""
	local memory_check_page=""
	local postgresql_warning_page=""

	DEFAULT_RESTORE=true
	local version="$(echo "$SYNOPKG_OLD_PKGVER" | cut -d- -f2)"

	# 2GB and 4GB check
	if memory_lt 1800; then
		memory_check_page="$(PageMemoryCheck false)"
	elif memory_lt 3800; then
		memory_check_page="$(PageMemoryCheck true)"
	fi

	if need_convert_db "$version"; then
		postgresql_warning_page="$(PagePostegresqlWarning)"
	fi

	upgrade_page=$(page_append "$upgrade_page" "$memory_check_page")
	upgrade_page=$(page_append "$upgrade_page" "$postgresql_warning_page")
	echo "[$upgrade_page]" > "${SYNOPKG_TEMP_LOGFILE}"
	return 0
}

main "$@"
